import pytest

from huscy.project_ethics.services import create_ethic

pytestmark = pytest.mark.django_db


def test_create_ethic(project):
    result = create_ethic(project)

    assert result.project == project
    assert result.ethic_board is None
    assert result.code == ''


def test_create_with_ethic_board(project, ethic_board):
    result = create_ethic(project, ethic_board)

    assert result.project == project
    assert result.ethic_board is ethic_board
    assert result.code == ''


def test_create_with_code(project):
    result = create_ethic(project, code='1234567890')

    assert result.project == project
    assert result.ethic_board is None
    assert result.code == '1234567890'


def test_create_with_ethic_board_and_code(project, ethic_board):
    result = create_ethic(project, ethic_board, '1234567890')

    assert result.project == project
    assert result.ethic_board is ethic_board
    assert result.code == '1234567890'



from itertools import cycle

import pytest
from model_bakery import baker

from huscy.project_ethics.services import get_ethic_boards

pytestmark = pytest.mark.django_db


def test_ordering():
    baker.make('project_ethics.EthicBoard', name=cycle('A C D B'.split()), _quantity=4)

    result = get_ethic_boards()

    assert ['A', 'B', 'C', 'D'] == [ethic_board.name for ethic_board in result]



from django.shortcuts import get_object_or_404
from rest_framework import mixins, viewsets
from rest_framework.permissions import DjangoModelPermissions, IsAuthenticated

from huscy.projects.models import Project
from huscy.project_ethics import serializer, services
from huscy.project_ethics.models import Ethic
from huscy.project_ethics.services import get_ethic_files


class EthicViewSet(mixins.CreateModelMixin, mixins.DestroyModelMixin, mixins.UpdateModelMixin,
                   viewsets.GenericViewSet):
    permission_classes = (IsAuthenticated, )
    serializer_class = serializer.EthicSerializer

    def initial(self, request, *args, **kwargs):
        self.project = get_object_or_404(Project, pk=self.kwargs['project_pk'])
        super().initial(request, *args, **kwargs)

    def get_queryset(self):
        return services.get_or_create_ethics(self.project)

    def get_serializer_context(self):
        context = super().get_serializer_context()
        context['project'] = self.project
        return context


class EthicBoardViewSet(mixins.CreateModelMixin, mixins.DestroyModelMixin, mixins.ListModelMixin,
                        viewsets.GenericViewSet):
    permission_classes = (DjangoModelPermissions, )
    queryset = services.get_ethic_boards()
    serializer_class = serializer.EthicBoardSerializer


class EthicFileViewSet(mixins.CreateModelMixin, mixins.DestroyModelMixin, viewsets.GenericViewSet):
    permission_classes = (IsAuthenticated, )
    serializer_class = serializer.EthicFileSerializer

    def initial(self, request, *args, **kwargs):
        self.ethic = get_object_or_404(
            Ethic, pk=self.kwargs['ethic_pk'], project=self.kwargs['project_pk']
        )
        super().initial(request, *args, **kwargs)

    def get_queryset(self):
        return get_ethic_files(self.ethic)



from rest_framework import serializers

from huscy.project_ethics import models, services


class EthicBoardSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.EthicBoard
        fields = (
            'id',
            'name',
        )


class EthicFileSerializer(serializers.ModelSerializer):
    filetype_name = serializers.SerializerMethodField()

    class Meta:
        model = models.EthicFile
        fields = (
            'ethic',
            'filehandle',
            'filename',
            'filetype',
            'filetype_name',
            'uploaded_at',
            'uploaded_by',
        )
        read_only_fields = 'filename',

    def get_filetype_name(self, ethic_file):
        return ethic_file.get_filetype_display()

    def create(self, validated_data):
        creator = self.context.get('request').user
        return services.create_ethic_file(**validated_data, creator=creator)


class EthicSerializer(serializers.ModelSerializer):
    ethic_board_name = serializers.SerializerMethodField()
    ethic_files = EthicFileSerializer(many=True, read_only=True)

    class Meta:
        model = models.Ethic
        fields = (
            'code',
            'ethic_board',
            'ethic_board_name',
            'ethic_files',
            'id',
            'project',
        )
        read_only_fields = 'project',

    def get_ethic_board_name(self, ethic):
        return ethic.ethic_board and ethic.ethic_board.name

    def create(self, validated_data):
        project = self.context['project']
        return services.create_ethic(project, **validated_data)



from rest_framework.reverse import reverse
from rest_framework.status import HTTP_405_METHOD_NOT_ALLOWED


def test_retrieve_is_not_allowed(client, project, ethic, ethic_file):
    response = client.get(
        reverse(
            'ethicfile-detail',
            kwargs=dict(project_pk=project.pk, ethic_pk=ethic.pk, pk=ethic_file.id)
        )
    )

    assert response.status_code == HTTP_405_METHOD_NOT_ALLOWED


def test_list_is_not_allowed(client, project, ethic):
    response = client.get(
        reverse('ethicfile-list', kwargs=dict(project_pk=project.pk, ethic_pk=ethic.pk))
    )

    assert response.status_code == HTTP_405_METHOD_NOT_ALLOWED



from rest_framework.reverse import reverse
from rest_framework.status import HTTP_405_METHOD_NOT_ALLOWED


def test_retrieve_ethics_is_not_provided(client, ethic):
    response = client.get(
        reverse('ethic-detail', kwargs=dict(project_pk=ethic.project.pk, pk=ethic.pk))
    )

    assert response.status_code == HTTP_405_METHOD_NOT_ALLOWED


def test_list_ethics_is_not_provided(client, project):
    response = client.get(reverse('ethic-list', kwargs=dict(project_pk=project.pk)))

    assert response.status_code == HTTP_405_METHOD_NOT_ALLOWED
